import 'dart:async';
import 'dart:io';
import 'package:ecp_sync_plugin_example/Database/CreateQuery.dart';
import 'package:ecp_sync_plugin_example/Database/TableConstants.dart';
import 'package:ecp_sync_plugin_example/Database/ColumnConstants.dart';
import 'package:ecp_sync_plugin_example/models/Label_Mst.dart';
import 'package:ecp_sync_plugin_example/models/Login.dart';
import 'package:ecp_sync_plugin_example/models/Visit_Date_Setting.dart';
import 'package:path/path.dart';
import 'package:sqflite/sqflite.dart';
import 'package:path_provider/path_provider.dart';

class DBAdapter extends CreateQuery {
  static final DATABASE_NAME = "ecp_mobile_flutter.db";
  static final DBAdapter _bookDatabase = new DBAdapter._internal();
  Database db;
  bool didInit = false;

  static DBAdapter get() {
    return _bookDatabase;
  }

  DBAdapter._internal();

  /// Use this method to access the database, because initialization of the database (it has to go through the method channel)
  Future<Database> _getDb() async {
    if (!didInit) await _init();
    return db;
  }

  Future init() async {
    return await _init();
  }

  Future _init() async {
    // Get a location using path_provider
    Directory documentsDirectory = await getApplicationDocumentsDirectory();
    String path = join(documentsDirectory.path, DATABASE_NAME);
    db = await openDatabase(path, version: 1,
        onCreate: (Database db, int version) async {
      // When creating the db, create the table
      await db.execute(TABLE_CREATE_LABEL_MST);
      await db.execute(TABLE_CREATE_LOGIN);
      await db.execute(TABLE_CREATE_LABEL_MST);
      await db.execute(TABLE_CREATE_LoginDetails);
      await db.execute(TABLE_CREATE_Doctor_Details);
      await db.execute(TABLE_CREATE_TourPlan_Doctor);
      await db.execute(TABLE_CREATE_SMS_System);
      await db.execute(TABLE_CREATE_Dr_GiftSample);
      await db.execute(TABLE_CREATE_Module_Error_Log);
      //await db.execute(Constant.DROP + TableConstants.TABLE_City_Mst);
      await db.execute(TABLE_CREATE_City_Mst);
      await db.execute(TABLE_CREATE_Employee_Mst);
      await db.execute(TABLE_CREATE_Division_Mst);
      await db.execute(TABLE_CREATE_Geo_Mst);
      await db.execute(TABLE_CREATE_Dr_Category);
      await db.execute(TABLE_CREATE_Medical_Speciality);
      await db.execute(TABLE_CREATE_Product_Mst);
      await db.execute(TABLE_CREATE_Gift_Mst);
      await db.execute(TABLE_CREATE_SMS_ReasonCode);
      await db.execute(TABLE_CREATE_Designation_Mst);
      await db.execute(TABLE_CREATE_Stockist_Mst);
      await db.execute(TABLE_CREATE_Stockist_Visits);
      await db.execute(TABLE_CREATE_Chemist_Category);
      await db.execute(TABLE_CREATE_Chemist_Mst);
      await db.execute(TABLE_CREATE_Chemist_Visits);
      await db.execute(TABLE_CREATE_Chemist_POB);
      await db.execute(TABLE_CREATE_Chemist_POB_Details);
      await db.execute(TABLE_CREATE_Visit_Date_Setting);
      // await db.execute(TABLE_CREATE_MSLReport);
      await db.execute(TABLE_CREATE_CallAverageReport);
      await db.execute(TABLE_CREATE_Doctor_Visit);
      // await db.execute(TABLE_CREATE_Last_Doctor_Visit);
      // await db.execute(TABLE_CREATE_Last_Dr_GiftSample);
      //await db.execute(TABLE_CREATE_Last_Dr_VisitRemarks);
      // await db.execute(TABLE_CREATE_Dr_Support);
      await db.execute(TABLE_CREATE_TourPlan_Set);
      await db.execute(TABLE_CREATE_TourPlan_Type);
      await db.execute(TABLE_CREATE_TourPlan_SubmitSet);
      await db.execute(TABLE_CREATE_TourPlan_SubmitEmpConfig);
      await db.execute(TABLE_CREATE_TourPlan_SetDetails);
      await db.execute(TABLE_CREATE_TourPlan_SetDetails_Android);
      await db.execute(TABLE_CREATE_TourPlan_Final);
      // await db.execute(TABLE_CREATE_Dr_Product);
      await db.execute(TABLE_CREATE_SecondaryDaysLimit);
      await db.execute(TABLE_CREATE_Upload_DisplayColumn_Config);
      await db.execute(TABLE_CREATE_State_Wise_PTS);
      await db.execute(TABLE_CREATE_Secondary_Data_Approval_Details);
      await db.execute(TABLE_CREATE_Secondary_Data);
      await db.execute(TABLE_CREATE_Secondary_Data_PreApproved);
      // await db.execute(TABLE_CREATE_Dr_Support_Entry);
      // await db.execute(TABLE_CREATE_Dr_Product_Support);
      await db.execute(TABLE_CREATE_Visual_Aids_Type_Mst);
      await db.execute(TABLE_CREATE_Visual_Aids_Type_Details);
      await db.execute(TABLE_CREATE_Visual_Aids_Mst);
      await db.execute(TABLE_CREATE_Visual_Aids_MappingWithType);
      await db.execute(TABLE_CREATE_Visual_Aids_Dr_Feedback);
      await db.execute(TABLE_CREATE_Emp_Joint_Work_Detail);
      await db.execute(TABLE_CREATE_LOCKINGDAYSLIMIT);
      await db.execute(TABLE_CREATE_VISUAL_AIDS_DR_FEEDBACK_DAYWISE);
      /*Hospital Table Create*/
      await db.execute(TABLE_CREATE_Dr_Group_Type);
      await db.execute(TABLE_CREATE_Dr_Group_Mst);
      await db.execute(TABLE_CREATE_Dr_Group_Mapping);
      await db.execute(TABLE_CREATE_Product_Group);
      await db.execute(TABLE_CREATE_Campaign_Mst);
      // await db.execute(TABLE_Weekly_ProductGroup_Secondary);
      await db.execute(CREATE_TABLE_Emp_Child_Employees);
      await db.execute(CREATE_TABLE_State_Mst);
      await db.execute(TABLE_CREATE_Doctor_POB);
      await db.execute(TABLE_CREATE_Doctor_POB_Details);
      await db.execute(TABLE_CREATE_chemist_GiftSample);
      await db.execute(TABLE_CREATE_Tour_Plan_Group);
      await db.execute(TABLE_CREATE_TourPlan_SubmitEmpList);
      await db.execute(TABLE_CREATE_Stockist_POB);
      await db.execute(TABLE_CREATE_Stockist_POB_Details);
      await db.execute(TABLE_CREATE_Stockist_GiftSample);
      await db.execute(TABLE_CREATE_Customer_Collection_Details);
      await db.execute(TABLE_CREATE_ReportingDatesWithCriteria);
      await db.execute(TABLE_CREATE_SMS_Extra_Dr_Visit);
      await db.execute(TABLE_CREATE_Various_Type_Mst);
      await db.execute(TABLE_CREATE_Holiday_Mst);
      // await db.execute(TABLE_CREATE_GlucoMeter_Plan);
      // await db.execute(TABLE_CREATE_GlucoMeter_Visit);
      // await db.execute(TABLE_CREATE_GlucoMeter_Visit_Details);
      //added this table in hospital model
      await db.execute(TABLE_CREATE_Dr_Group_Visit_Purpose_Mst);
      await db.execute(TABLE_CREATE_Stage_Mst);
      await db.execute(TABLE_CREATE_Input_Required_Mst);
      await db.execute(TABLE_CREATE_Product_Line_Mst);
      await db.execute(TABLE_CREATE_Dr_Group_Hosp_Visit);
      await db.execute(TABLE_CREATE_Dr_Group_Visit_Brand_Camp_Details);
      await db.execute(TABLE_CREATE_Dr_Group_Change_Prescriber);
      await db.execute(TABLE_CREATE_Dr_Group_TPPlan);
      await db.execute(TABLE_CREATE_Dr_Cat_Restriction);
      await db.execute(TABLE_CREATE_Dr_Cat_Restriction_DayTypeWise);
      await db.execute(TABLE_CREATE_TourPlan_MTP_SetDetails);
      await db.execute(TABLE_CREATE_Customisation_Master);
      await db.execute(TABLE_CREATE_Customisation_Image);
      await db.execute(CREATE_TABLE_GPSLocation_Log);
      await db.execute(CREATE_TABLE_Chemist_Product_Stock);
      // await db.execute(CREATE_TABLE_GlucoMeter_Visit_IportDetails);
      await db.execute(CREATE_TABLE_APP_Version_Mst);
      await db.execute(CREATE_TABLE_Emp_LocationTrack);
      await db.execute(CREATE_TABLE_DayPunch_Location);
      await db.execute(CREATE_TABLE_Mobile_Process_Log);
      await db.execute(CREATE_TABLE_Mobile_Menu_AccessLog);
      await db.execute(TABLE_CREATE_Dr_Request_Parameter_Config);

      //await db.execute(DROP + TableConstants.TABLE_APK_Details);
      await db.execute(CREATE +
          TableConstants.TABLE_APK_Details +
          "( " +
          ColumnConstants.COL_intGlCode +
          " " +
          "INTEGER PRIMARY " +
          "KEY, " +
          ColumnConstants.COL_varCurrentVersion +
          " TEXT, " +
          ColumnConstants.COL_dtUpdatedDate +
          "" +
          " DATETIME," +
          " " +
          ColumnConstants.COL_chrUpdatedWeb +
          " CHAR )");
      await db.execute(TABLE_CREATE_Stockiest_Wise_PTS);
    });
    didInit = true;
  }

  Future clearLabelMst() async {
    var db = await _getDb();
    await db.delete(TableConstants.TABLE_LABEL_MST);
  }

  Future clearVisitDateSetting() async {
    var db = await _getDb();
    await db.delete(TableConstants.TABLE_Visit_Date_Setting);
  }

  Future<bool> getCountLogin() async {
    var db = await _getDb();
    List<Map> list =
        await db.rawQuery('Select * from ${TableConstants.TABLE_Login}', []);
    if (list != null) {
      return true;
    } else {
      return false;
    }
  }

  Future insertLogin(Login book, String username, String password,
      String mobile, String imei) async {
    var db = await _getDb();
    await db.rawInsert(
        'INSERT OR REPLACE INTO '
        '${TableConstants.TABLE_Login} (${ColumnConstants.COLUMN_intGlCode}, '
        '${ColumnConstants.COLUMN_varUserName}, '
        '${ColumnConstants.COLUMN_varPassword}, '
        '${ColumnConstants.COLUMN_varPersonName}, '
        '${ColumnConstants.COLUMN_fk_PersonGlCode}, '
        '${ColumnConstants.COLUMN_varMobileNumber}, '
        '${ColumnConstants.COLUMN_varClientName}, '
        '${ColumnConstants.COLUMN_varIsAuthorise}, '
        '${ColumnConstants.COLUMN_varIsSmson}, '
        '${ColumnConstants.COLUMN_varImei})'
        ' VALUES(?,?,?,?,?,?,?,?,?,?)',
        [
          1,
          username,
          password,
          book.varFullName,
          book.fk_PersonGlCode,
          mobile,
          book.Client,
          book.IsAuthorised,
          book.IsSMSOn,
          imei
        ]);
  }

  Future updateLoginPassword(Login book, String password, int intGlCode) async {
    var db = await _getDb();
    await db.rawUpdate(
        'Update ${TableConstants.TABLE_Login} SET ${ColumnConstants.COLUMN_varPassword} = ?, ${ColumnConstants.COLUMN_varIsAuthorise} = ? WHERE ${ColumnConstants.COLUMN_intGlCode} = ?',
        [password, book.IsAuthorised, intGlCode]);
  }

  Future insertLabelMst(Label_Mst book) async {
    var db = await _getDb();
    await db.rawInsert(
        'INSERT OR REPLACE INTO '
        '${TableConstants.TABLE_LABEL_MST} (${ColumnConstants.COLUMN_varType}, ${ColumnConstants.COLUMN_varDisplayName}, ${ColumnConstants.COLUMN_varShortName})'
        ' VALUES(?, ?, ?)',
        [book.varType, book.varDisplayName, book.varShortName]);
  }

  Future insertVisitDateSetting(Visit_Date_Setting book) async {
    var db = await _getDb();
    await db.rawInsert(
        'INSERT OR REPLACE INTO '
        '${TableConstants.TABLE_Visit_Date_Setting} (${ColumnConstants.COLUMN_intGLCode}, ${ColumnConstants.COLUMN_varVisitType}, ${ColumnConstants.COLUMN_intPrevDays},${ColumnConstants.COLUMN_intNextDays},${ColumnConstants.COLUMN_chrVisible})'
        ' VALUES(?, ?, ?, ?, ?)',
        [
          book.intGlCode,
          book.varVisitType,
          book.intPrevDays,
          book.intNextDays,
          book.chrVisible
        ]);
  }
}
