import 'dart:async';
import 'dart:io';
import 'package:ecp_sync_plugin_example/DatabaseQuery.dart';
import 'package:path/path.dart';
import 'package:sqflite/sqflite.dart';
import 'package:path_provider/path_provider.dart';

class DatabaseHelper {
  static final DATABASE_NAME = "BASF-HK-Database.db";
  static final DatabaseHelper _bookDatabase = new DatabaseHelper._internal();
  Database db;
  bool didInit = false;

  static DatabaseHelper get() {
    return _bookDatabase;
  }

  DatabaseHelper._internal();

  /// Use this method to access the database, because initialization of the database (it has to go through the method channel)
  Future<Database> _getDb() async {
    if (!didInit) await _init();
    return db;
  }

  Future init() async {
    return await _init();
  }

  Future _init() async {
    // Get a location using path_provider
    Directory documentsDirectory = await getApplicationDocumentsDirectory();
    String path = join(documentsDirectory.path, DATABASE_NAME);
    db = await openDatabase(path, version: 1,
        onCreate: (Database db, int version) async {
      // When creating the db, create the table
      await db.execute(DatabaseQuery.CREATE_TABLE_CPM_CUSTOMER_DISPATCH);
      await db.execute(
          DatabaseQuery.CREATE_TABLE_CPM_CUSTOMER_DISPATCH_PRODUCT_DETAILS);
      await db.execute(DatabaseQuery.CREATE_TABLE_CPM_CUSTOMER_RECEIVE);
      await db.execute(
          DatabaseQuery.CREATE_TABLE_CPM_CUSTOMER_RECEIVE_PRODUCT_DETAILS);
      await db.execute(DatabaseQuery.CREATE_TABLE_CPM_STRICKER_Details);
      await db.execute(DatabaseQuery.CREATE_TABLE_STATUS_MST);
      await db.execute(DatabaseQuery.CREATE_TABLE_PRODUCT_SKU_MST);
      await db.execute(DatabaseQuery.CREATE_TABLE_COUNTRY_MST);
      await db.execute(
          DatabaseQuery.CREATE_TABLE_POLITICAL_GEOGRAPHY_LEVEL_REGIONAL_MST);
      await db.execute(
          DatabaseQuery.CREATE_TABLE_POLITICAL_GEOGRAPHY_LEVEL_COUNTRY_MST);
      await db.execute(DatabaseQuery.CREATE_TABLE_POLITICAL_GEOGRAPHY_DETAILS);
      await db.execute(DatabaseQuery.CREATE_TABLE_SYSTEM_CONFIGURATION_MST);
      await db.execute(DatabaseQuery.CREATE_TABLE_EMPLOYEE_TYPE_MST);
      await db.execute(DatabaseQuery.CREATE_TABLE_CUSTOMER_TYPE_REGIONAL_MST);
      await db.execute(DatabaseQuery.CREATE_TABLE_CUSTOMER_TYPE_COUNTRY_MST);
      await db.execute(DatabaseQuery.CREATE_TABLE_PERSON_MST);
      await db.execute(DatabaseQuery.CREATE_TABLE_UOM_MST);
      await db.execute(DatabaseQuery.CREATE_TABLE_MENU_MST);
      await db.execute(DatabaseQuery.CREATE_TABLE_PSKU_BATCH_STOCK);
      await db.execute(DatabaseQuery.CREATE_TABLE_CUSTOMER_PARTNER_MST);
      await db.execute(DatabaseQuery.CREATE_TABLE_CPM_CUSTOMER_STICKER_DAMAGE);
      await db.execute(DatabaseQuery.CREATE_TABLE_LOGIN_DETAILS);
      await db.execute(
          DatabaseQuery.CREATE_TABLE_CPM_CUSTOMER_STICKER_DAMAGE_REMOVE);

      String INDEX_QUERY =
          "CREATE INDEX idx_btc_out_sticker ON cpm_sticker_details (varBTC_No_Out)";
      await db.execute(INDEX_QUERY);

      INDEX_QUERY =
          "CREATE INDEX idx_qr_code_out_sticker ON cpm_sticker_details (varQR_Code_Out)";
      await db.execute(INDEX_QUERY);

      INDEX_QUERY =
          "CREATE INDEX idx_CPM_Customer_Dispatch_varDONO ON CPM_Customer_Dispatch (varDONO)";
      await db.execute(INDEX_QUERY);

      INDEX_QUERY =
          "CREATE INDEX idx_CPM_Customer_Dispatch_fk_Customer_From_GlCode ON CPM_Customer_Dispatch (fk_Customer_From_GlCode)";
      await db.execute(INDEX_QUERY);

      INDEX_QUERY =
          "CREATE INDEX idx_CPM_Customer_Dispatch_fk_Customer_To_GlCode ON CPM_Customer_Dispatch (fk_Customer_To_GlCode)";
      await db.execute(INDEX_QUERY);

      INDEX_QUERY =
          "CREATE INDEX idx_CPM_Customer_Dispatch_fk_Product_SKU_Glcode ON CPM_Customer_Dispatch_Product_Details (fk_Product_SKU_Glcode)";
      await db.execute(INDEX_QUERY);

      INDEX_QUERY =
          "CREATE INDEX idx_CPM_Customer_Dispatch_fk_Customer_DispatchGlCode ON CPM_Customer_Dispatch_Product_Details (fk_Customer_DispatchGlCode)";
      await db.execute(INDEX_QUERY);

      INDEX_QUERY =
          "CREATE INDEX idx_CPM_Customer_Dispatch_fk_StickerGlCode ON CPM_Customer_Dispatch_Product_Details (fk_StickerGlCode)";
      await db.execute(INDEX_QUERY);

      INDEX_QUERY =
          "CREATE INDEX idx_CPM_Customer_Dispatch_varSticker ON CPM_Customer_Dispatch_Product_Details (varSticker)";
      await db.execute(INDEX_QUERY);

      INDEX_QUERY =
          "CREATE INDEX idx_CPM_Customer_Receive_Product_Details_varSticker ON CPM_Customer_Receive_Product_Details (varSticker)";
      await db.execute(INDEX_QUERY);

      INDEX_QUERY =
          "CREATE INDEX idx_CPM_Customer_Sticker_Damage_varSticker ON CPM_Customer_Sticker_Damage (varSticker)";
      await db.execute(INDEX_QUERY);

      INDEX_QUERY =
          "CREATE INDEX idx_CPM_Customer_Receive_fk_DispatchGlCode ON CPM_Customer_Receive (fk_DispatchGlCode)";
      await db.execute(INDEX_QUERY);

      INDEX_QUERY =
          "CREATE INDEX idx_CPM_Customer_Receive_fk_Customer_From_GlCode ON CPM_Customer_Receive (fk_Customer_From_GlCode)";
      await db.execute(INDEX_QUERY);

      INDEX_QUERY =
          "CREATE INDEX idx_CPM_Customer_Receive_fk_Customer_To_GlCode ON CPM_Customer_Receive (fk_Customer_To_GlCode)";
      await db.execute(INDEX_QUERY);

      INDEX_QUERY =
          "CREATE INDEX idx_CPM_Customer_Receive_Product_Details_fk_Product_SKU_Glcode ON CPM_Customer_Receive_Product_Details (fk_Product_SKU_Glcode)";
      await db.execute(INDEX_QUERY);

      INDEX_QUERY =
          "CREATE INDEX idx_CPM_Customer_Receive_Product_Details_fk_Customer_ReceiveGlCode ON CPM_Customer_Receive_Product_Details (fk_Customer_ReceiveGlCode)";
      await db.execute(INDEX_QUERY);

      INDEX_QUERY =
          "CREATE INDEX idx_CPM_Customer_Receive_Product_Details_fk_StickerGlCode ON CPM_Customer_Receive_Product_Details (fk_StickerGlCode)";
      await db.execute(INDEX_QUERY);

      INDEX_QUERY =
          "CREATE INDEX idx_CPM_Customer_Sticker_Damage_fk_CustomerGlCode ON CPM_Customer_Sticker_Damage (fk_CustomerGlCode)";
      await db.execute(INDEX_QUERY);

      INDEX_QUERY =
          "CREATE INDEX idx_CPM_Customer_Sticker_Damage_fk_StickerGlCode ON CPM_Customer_Sticker_Damage (fk_StickerGlCode)";
      await db.execute(INDEX_QUERY);

      INDEX_QUERY =
          "CREATE INDEX idx_CPM_Sticker_Details_fk_CustomerGlCode ON CPM_Sticker_Details (fk_CustomerGlCode)";
      await db.execute(INDEX_QUERY);

      INDEX_QUERY =
          "CREATE INDEX idx_CPM_Sticker_Details_fk_Product_SKUGlCode ON CPM_Sticker_Details (fk_Product_SKUGlCode)";
      await db.execute(INDEX_QUERY);

      INDEX_QUERY =
          "CREATE INDEX idx_CPM_Sticker_Details_varBatch_No ON CPM_Sticker_Details (varBatch_No)";
      await db.execute(INDEX_QUERY);

      INDEX_QUERY =
          "CREATE INDEX idx_CPM_Sticker_Details_fk_Sticker_GeneratedBy ON CPM_Sticker_Details (fk_Sticker_GeneratedBy)";
      await db.execute(INDEX_QUERY);

      INDEX_QUERY =
          "CREATE INDEX idx_CPM_Sticker_Details_fk_Last_DispatchedTo ON CPM_Sticker_Details (fk_Last_DispatchedTo)";
      await db.execute(INDEX_QUERY);

      INDEX_QUERY =
          "CREATE INDEX idx_CPM_Sticker_Details_fk_Last_DispatchedBy ON CPM_Sticker_Details (fk_Last_DispatchedBy)";
      await db.execute(INDEX_QUERY);

      String CREATE_TEMP =
          "CREATE TABLE CPM_TEMP_BULK(intGlCode INTEGER  NOT NULL PRIMARY KEY AUTOINCREMENT,intRowNo INTEGER,stickerNo TEXT,stickerId TEXT,chrValid TEXT)";
      await db.execute(CREATE_TEMP);
    });
    didInit = true;
  }

  Future<bool> getCountLogin() async {
    var db = await _getDb();
    List<Map> list = await db.rawQuery('Select * from Login', []);
    if (list != null) {
      return true;
    } else {
      return false;
    }
  }

  /*Future clearLabelMst() async {
    var db = await _getDb();
    await db.delete(TableConstants.TABLE_LABEL_MST);
  }

  Future clearVisitDateSetting() async {
    var db = await _getDb();
    await db.delete(TableConstants.TABLE_Visit_Date_Setting);
  }



  Future insertLogin(Login book, String username, String password,
      String mobile, String imei) async {
    var db = await _getDb();
    await db.rawInsert(
        'INSERT OR REPLACE INTO '
            '${TableConstants.TABLE_Login} (${ColumnConstants.COLUMN_intGlCode}, '
            '${ColumnConstants.COLUMN_varUserName}, '
            '${ColumnConstants.COLUMN_varPassword}, '
            '${ColumnConstants.COLUMN_varPersonName}, '
            '${ColumnConstants.COLUMN_fk_PersonGlCode}, '
            '${ColumnConstants.COLUMN_varMobileNumber}, '
            '${ColumnConstants.COLUMN_varClientName}, '
            '${ColumnConstants.COLUMN_varIsAuthorise}, '
            '${ColumnConstants.COLUMN_varIsSmson}, '
            '${ColumnConstants.COLUMN_varImei})'
            ' VALUES(?,?,?,?,?,?,?,?,?,?)',
        [
          1,
          username,
          password,
          book.varFullName,
          book.fk_PersonGlCode,
          mobile,
          book.Client,
          book.IsAuthorised,
          book.IsSMSOn,
          imei
        ]);
  }

  Future updateLoginPassword(Login book, String password, int intGlCode) async {
    var db = await _getDb();
    await db.rawUpdate(
        'Update ${TableConstants.TABLE_Login} SET ${ColumnConstants.COLUMN_varPassword} = ?, ${ColumnConstants.COLUMN_varIsAuthorise} = ? WHERE ${ColumnConstants.COLUMN_intGlCode} = ?',
        [password, book.IsAuthorised, intGlCode]);
  }

  Future insertLabelMst(Label_Mst book) async {
    var db = await _getDb();
    await db.rawInsert(
        'INSERT OR REPLACE INTO '
            '${TableConstants.TABLE_LABEL_MST} (${ColumnConstants.COLUMN_varType}, ${ColumnConstants.COLUMN_varDisplayName}, ${ColumnConstants.COLUMN_varShortName})'
            ' VALUES(?, ?, ?)',
        [book.varType, book.varDisplayName, book.varShortName]);
  }

  Future insertVisitDateSetting(Visit_Date_Setting book) async {
    var db = await _getDb();
    await db.rawInsert(
        'INSERT OR REPLACE INTO '
            '${TableConstants.TABLE_Visit_Date_Setting} (${ColumnConstants.COLUMN_intGLCode}, ${ColumnConstants.COLUMN_varVisitType}, ${ColumnConstants.COLUMN_intPrevDays},${ColumnConstants.COLUMN_intNextDays},${ColumnConstants.COLUMN_chrVisible})'
            ' VALUES(?, ?, ?, ?, ?)',
        [
          book.intGlCode,
          book.varVisitType,
          book.intPrevDays,
          book.intNextDays,
          book.chrVisible
        ]);
  }*/
}
